﻿// VideoGameCasePhysics Version 1 - Using Papervision 2.0 Beta - Revision 904, JigLibFlash - Revision 81, BulkLoader - Revision 257
// A very simple demo of dropping multiple video game cases on top of each other (to show off the updated JigLibFlash library)
//
// released under MIT License
// http://www.opensource.org/licenses/mit-license.php
//
//@author		Devin Reimer
//@version		1.0.0
//@website 		http://blog.almostlogical.com, http://www.permaven.com


//Copyright (c) 2009 Devin Reimer

/*
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com 
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Rectangle;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	import org.papervision3d.lights.PointLight3D;
	import org.papervision3d.materials.shadematerials.FlatShadeMaterial;
	import org.papervision3d.materials.WireframeMaterial;
	import org.papervision3d.view.BasicView;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.materials.BitmapMaterial;
	import org.papervision3d.materials.MovieMaterial;
	import org.papervision3d.view.layer.ViewportLayer;
	import org.papervision3d.view.layer.util.ViewportLayerSortMode;

	import br.com.stimuli.loading.BulkLoader;
	import br.com.stimuli.loading.BulkProgressEvent;
	
	import jiglib.geometry.*;
	import jiglib.math.*;
	import jiglib.cof.JConfig;
	import jiglib.physics.*;
	import jiglib.physics.constraint.*;
	import jiglib.plugin.papervision3d.*;
	
	public class VideoGameCasePhysics extends Sprite
	{
		//private static const LIVE_PATH:String = "/workingExamples/VideoGameCasePhysics/";
		private static const LIVE_PATH:String = "";
		private static const STANDARD_COVER_PATH:String = LIVE_PATH + "Images/StandardCover/";
		private static const BACK_COVER_OVERLAY_FILE_PATH:String = STANDARD_COVER_PATH + "BackCover.png";
		private static const COVER_OVERLAY_FILE_PATH:String = STANDARD_COVER_PATH + "Cover.png";
		private static const SPINE_OVERLAY_FILE_PATH:String = STANDARD_COVER_PATH + "Spine.png";
		private static const OPEN_END_IMAGE_FILE_PATH:String = STANDARD_COVER_PATH + "OpenEnd.jpg";
		private static const TOP_AND_BOTTOM_IMAGE_FILE_PATH:String = STANDARD_COVER_PATH + "TopAndBottom.jpg";
		private static const VIDEO_GAME_CASE_IMAGE_FILE_PATH:String = LIVE_PATH + "Images/Covers/CaseScanSmall.jpg";  //add your own case scans here (Scan dimensions are 325 x 218)
		
		private static const CASE_WIDTH:int = 158;
		private static const CASE_DEPTH:int = 17;
		private static const CASE_HEIGHT:int = 226;
		private static const MAX_NUM_OF_CASES:int = 7;
		
		public static const toRADIANS:Number = Math.PI / 180;
			
		private var view:BasicView;
		private var videoCaseRigidBody:RigidBody;
		private var mainBulkLoader:BulkLoader;
		
		private var physics:Papervision3DPhysics;
		
		private var timeCounter:int = 0; //counts intervals
		
		private var videoCaseMaterials:MaterialsList;
		private var caseArray:Array = new Array();
		
		public function VideoGameCasePhysics() 
		{		
			var blogText:TextField = new TextField();
			blogText.defaultTextFormat = new TextFormat("_sans", 10, 0x000000, true);
			blogText.text = "blog.almostlogical.com - Devin Reimer";
			blogText.autoSize = TextFieldAutoSize.LEFT;
			blogText.x = stage.stageWidth - (blogText.textWidth+ 8);
			blogText.y = stage.stageHeight - (blogText.textHeight + 12);
			//draw white background
			this.graphics.beginFill(0xE6EBFF);
			this.graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
			this.graphics.endFill();
				
			view = new BasicView(stage.stageWidth, stage.stageHeight);
			
			loadAssets();
			
			view.camera.z += 450;
			view.camera.y += 100;
			
			addChild(view);
			addChild(blogText);
			
			view.viewport.containerSprite.sortMode = ViewportLayerSortMode.INDEX_SORT;
		}
		
		private function beginRender():void
		{
			addEventListener(Event.ENTER_FRAME, loop);
		}
		
		private function loadAssets():void
		{
			mainBulkLoader = new BulkLoader("MainBulkLoader");
			mainBulkLoader.add(BACK_COVER_OVERLAY_FILE_PATH);
			mainBulkLoader.add(COVER_OVERLAY_FILE_PATH);
			mainBulkLoader.add(SPINE_OVERLAY_FILE_PATH);
			mainBulkLoader.add(OPEN_END_IMAGE_FILE_PATH);
			mainBulkLoader.add(TOP_AND_BOTTOM_IMAGE_FILE_PATH);
			mainBulkLoader.add(VIDEO_GAME_CASE_IMAGE_FILE_PATH);
			mainBulkLoader.addEventListener(BulkProgressEvent.COMPLETE,loadAssetsComplete);
			mainBulkLoader.start();
		}
		
		private function loadAssetsComplete(evt:Event):void
		{
			physics = new Papervision3DPhysics(view.scene, 3);
			createScene();
		}
		
		private function createScene():void
		{	
			var coverSprite:Sprite = createSpriteCover(VIDEO_GAME_CASE_IMAGE_FILE_PATH); 
		
			videoCaseMaterials = new MaterialsList();
			videoCaseMaterials.addMaterial(new MovieMaterial(coverSprite, false, false, true, new Rectangle(0, 0, CASE_WIDTH, CASE_HEIGHT)),"back");
			videoCaseMaterials.addMaterial(new MovieMaterial(coverSprite, false, false, true, new Rectangle(CASE_WIDTH, 0, CASE_DEPTH, CASE_HEIGHT)),"right");
			videoCaseMaterials.addMaterial(new MovieMaterial(coverSprite, false, false, true, new Rectangle(CASE_WIDTH + CASE_DEPTH, 0, CASE_WIDTH, CASE_HEIGHT)),"front");	
			videoCaseMaterials.addMaterial(new BitmapMaterial(mainBulkLoader.getBitmapData(OPEN_END_IMAGE_FILE_PATH),true), "left");
			videoCaseMaterials.addMaterial(new BitmapMaterial(mainBulkLoader.getBitmapData(TOP_AND_BOTTOM_IMAGE_FILE_PATH),true), "top");
			videoCaseMaterials.addMaterial(new BitmapMaterial(mainBulkLoader.getBitmapData(TOP_AND_BOTTOM_IMAGE_FILE_PATH), true), "bottom");
				
			createVideoCase();
			createGround();			
			beginRender();
		}
		
		private function createGround():void
		{
			var ground:RigidBody;
			var mylight:PointLight3D = new PointLight3D(true, true);
			var flatShaderMat:FlatShadeMaterial = new FlatShadeMaterial(mylight, 0x77ee77);
			var materiaList:MaterialsList = new MaterialsList();
			
			mylight.y = 300;
			mylight.z = -400;
			
			materiaList.addMaterial(flatShaderMat, "all");	
			ground = physics.createCube(materiaList, 700, 500, 10, 2, 2, 2);
			ground.moveTo(new JNumber3D(0, -150, 0));
			ground.movable = false;
			ground.friction = 0.2;
			ground.restitution = 0.8;
			
			view.viewport.getChildLayer(physics.getMesh(ground)).layerIndex = 1;
		}
		
		private function createVideoCase():void
		{
			videoCaseRigidBody = physics.createCube(videoCaseMaterials, CASE_WIDTH, CASE_DEPTH, CASE_HEIGHT, 6, 6, 6);
			videoCaseRigidBody.rotationY = 180 * toRADIANS; //make front face towards the camera
			videoCaseRigidBody.moveTo(new JNumber3D((Math.random() * 200) - 100, 500, (Math.random() * 200) - 100)); //create random position above ground
			videoCaseRigidBody.rotationY = ((Math.random() * 360) - 180) * toRADIANS; //random y rotation
			videoCaseRigidBody.rotationX = ((Math.random() * 360) - 180) * toRADIANS; //random x roation
			videoCaseRigidBody.updateObject3D(); //position the papervision object (don't wait until physics step)
			
			view.viewport.getChildLayer(physics.getMesh(videoCaseRigidBody)).layerIndex = 2;
			
			caseArray.push(videoCaseRigidBody); //add to array
		}
		
		//create sprite of the camera
		private function createSpriteCover(coverFilePath:String):Sprite
		{
			var fullCoverSprite:Sprite = new Sprite();
			var backCoverBitmap:Bitmap;
			var spineBitmap:Bitmap;
			var frontCoverBitmap:Bitmap;
			var scanCoverBitmap:Bitmap;
			var buffer:int = 4;
		
			backCoverBitmap = mainBulkLoader.getBitmap(BACK_COVER_OVERLAY_FILE_PATH);
			spineBitmap = mainBulkLoader.getBitmap(SPINE_OVERLAY_FILE_PATH);
			frontCoverBitmap = mainBulkLoader.getBitmap(COVER_OVERLAY_FILE_PATH);
			scanCoverBitmap = mainBulkLoader.getBitmap(coverFilePath);
			
			spineBitmap.x = CASE_WIDTH;
			frontCoverBitmap.x = CASE_WIDTH + CASE_DEPTH;
			
			scanCoverBitmap.x = buffer
			scanCoverBitmap.y = buffer;
			
			fullCoverSprite.addChild(scanCoverBitmap);
			fullCoverSprite.addChild(spineBitmap);
			fullCoverSprite.addChild(frontCoverBitmap);	
			fullCoverSprite.addChild(backCoverBitmap);
				
			return fullCoverSprite;
		}
		
		private function loop(evt:Event):void
		{
			view.singleRender();
			physics.step();
				
			timeCounter++;
			if (timeCounter > 100) //every 100 intervals create a new both
			{
				if (PhysicsSystem.getInstance().bodys.length > MAX_NUM_OF_CASES) //if 7 cases already exists removal all before creating next video game case
				{
					resetAll();
				}
				createVideoCase();
				timeCounter = 0;
			}
		}
		
		//remove all video game cases
		private function resetAll():void
		{
			var rigidBodyRemovalArray:Array = new Array();
			
			for each (var rigidBody:RigidBody in caseArray)
			{
				PhysicsSystem.getInstance().removeBody(rigidBody);
				view.scene.removeChild(physics.getMesh(rigidBody));
			}
			
			caseArray = new Array(); //reset array
		}
		
	}
	
}